@@ -6,12 +6,15 @@ The Modality class stores the raw image data and its affine transformation, with
 """
 
 import os
+import json
+import glob
 import numpy as np
 import nibabel as nib
 import pandas as pd
 from deepvoxnet2.components.sample import Sample
 from PIL import Image
 from matplotlib import pyplot as plt
+from sklearn.model_selection import KFold
 
 
 class SortedDict(dict):
@@ -313,6 +316,9 @@ class Mirc(SortedDict):
             If the affine or shape of any selected modality is not equal to that of the first selected modality.
         """
 
+        if modality_ids is None:
+            modality_ids = self.get_modality_ids()
+
         modality_ids = modality_ids if isinstance(modality_ids, list) else [modality_ids]
         ns = ns if isinstance(ns, list) else [ns] * len(modality_ids)
         clippings = clippings if isinstance(clippings, list) else [clippings] * len(modality_ids)
@@ -334,13 +340,13 @@ class Mirc(SortedDict):
             for dataset_id in dataset_ids:
                 for case_id in self[dataset_id]:
                     for record_id in self[dataset_id][case_id]:
-                        reference_modality = self[dataset_id][case_id][record_id][modality_ids[0]].load()
+                        reference_shape, reference_affine = self[dataset_id][case_id][record_id][modality_ids[0]].shape, self[dataset_id][case_id][record_id][modality_ids[0]].affine
                         for i, modality_id in enumerate(modality_ids):
-                            modality = self[dataset_id][case_id][record_id][modality_id].load()
-                            assert np.allclose(modality.affine, reference_modality.affine, rtol=rtol, atol=atol), f"Affine of {dataset_id}-{case_id}-{record_id}-{modality_id} is not equal to the reference from {modality_ids[0]}."
-                            assert np.array_equal(reference_modality.shape[1:4], modality.shape[1:4])
-                            img_sizes.append(modality.shape[1:4])
-                            voxel_sizes.append([s.round(2) for s in np.linalg.norm(modality.affine[0][:3, :3], 2, axis=0)])
+                            modality_shape, modality_affine = self[dataset_id][case_id][record_id][modality_id].shape, self[dataset_id][case_id][record_id][modality_id].affine
+                            assert np.allclose(modality_affine, reference_affine, rtol=rtol, atol=atol), f"Affine of {dataset_id}-{case_id}-{record_id}-{modality_id} is not equal to the reference from {modality_ids[0]}."
+                            assert np.array_equal(reference_shape[1:4], modality_shape[1:4])
+                            img_sizes.append(modality_shape[1:4])
+                            voxel_sizes.append([s.round(2) for s in np.linalg.norm(modality_affine[0][:3, :3], 2, axis=0)])
 
             fig, axs = plt.subplots(3, 3, figsize=(9, 10))
             for i in range(3):
@@ -376,6 +382,11 @@ class Mirc(SortedDict):
             plt.suptitle(" + ".join(dataset_ids))
             plt.show()
 
+    @staticmethod
+    def from_nnUNetv2_dataset(*nnUNetv2_dataset_args, **nnUNetv2_dataset_kwargs):
+        train_dataset, val_dataset, test_dataset = Dataset.from_nnUNetv2_dataset(*nnUNetv2_dataset_args, **nnUNetv2_dataset_kwargs)
+        return Mirc(train_dataset), Mirc(val_dataset), Mirc(test_dataset)
+
 
 class Dataset(SortedDict):
     """A class to represent a dataset, which is a collection of cases.
@@ -422,6 +433,94 @@ class Dataset(SortedDict):
         assert case.case_id not in self
         self[case.case_id] = case
 
+    @staticmethod
+    def from_nnUNetv2_dataset(dataset_dir, fold="all", nb_folds=5, file_extension=".nii.gz"):
+        assert file_extension in [".nii", ".nii.gz"], "Currently we only support nifti files."
+        images_dir_trainval = os.path.join(dataset_dir, "imagesTr")
+        labels_dir_trainval = os.path.join(dataset_dir, "labelsTr")
+        images_dir_test = os.path.join(dataset_dir, "imagesTs")
+        labels_dir_test = os.path.join(dataset_dir, "labelsTs")
+        case_ids_trainval = ["_".join(filename.split("_")[:-1]) for filename in sorted(os.listdir(images_dir_trainval)) if not filename.startswith(".") and filename.endswith(file_extension)]
+        if os.path.isdir(images_dir_test):
+            case_ids_test = ["_".join(filename.split("_")[:-1]) for filename in sorted(os.listdir(images_dir_test)) if not filename.startswith(".") and filename.endswith(file_extension)]
+        
+        else:
+            case_ids_test = []
+
+        dataset_id = os.path.split(dataset_dir)[1]
+
+        if os.path.isfile(os.path.join(dataset_dir, "splits_final.json")):
+            splits_final_path = os.path.join(dataset_dir, "splits_final.json")
+        
+        elif os.path.isfile(os.path.join(dataset_dir, "splits_final.json").replace("nnUNet_raw", "nnUNet_preprocessed")):
+            splits_final_path = os.path.join(dataset_dir, "splits_final.json").replace("nnUNet_raw", "nnUNet_preprocessed")
+        
+        else:
+            splits_final_path = None
+        
+        if splits_final_path is not None:
+            with open(splits_final_path, "r") as f:
+                splits_final = json.load(f)
+
+            assert nb_folds is None or nb_folds == len(splits_final)
+            nb_folds = len(splits_final)
+        
+        if fold == "all":
+            train_dataset_id = f"{dataset_id}_train"
+            val_dataset_id = f"{dataset_id}_val"
+            case_ids_train = case_ids_trainval
+            case_ids_val = []
+        
+        else:
+            assert nb_folds is not None and nb_folds > 1 and fold in range(nb_folds), "Please make sure to specify a fold that's smaller than the total number of folds specified in nb_folds > 1."
+            train_dataset_id = f"{dataset_id}_train_{fold}"
+            val_dataset_id = f"{dataset_id}_val_{fold}"
+            if splits_final_path is None:
+                kfold = KFold(n_splits=nb_folds, shuffle=True, random_state=12345)
+                for fold_i, (train_idx, val_idx) in enumerate(kfold.split(case_ids_trainval)):
+                    if fold_i == fold:
+                        case_ids_train = [case_ids_trainval[i] for i in train_idx]
+                        case_ids_val = [case_ids_trainval[i] for i in val_idx]
+                        break
+            
+            else:
+                case_ids_train = splits_final[fold]["train"]
+                case_ids_val = splits_final[fold]["val"]
+                assert set(case_ids_trainval) == set(case_ids_train + case_ids_val)
+        
+        train_dataset, val_dataset = Dataset(train_dataset_id), Dataset(val_dataset_id)
+        for case_id in case_ids_trainval:
+            case = Case(case_id)
+            record = Record("record_0")
+            assert os.path.isfile(os.path.join(labels_dir_trainval, f"{case_id}{file_extension}"))
+            record.add(NiftiFileModality("label", os.path.join(labels_dir_trainval, f"{case_id}{file_extension}")))
+            for i, channel_path in enumerate(sorted(glob.glob(os.path.join(images_dir_trainval, f"{case_id}_*.nii.gz")))):
+                record.add(NiftiFileModality("channel_{:04}".format(i), channel_path))
+
+            case.add(record)
+            if case_id in case_ids_train:
+                train_dataset.add(case)
+            
+            else:
+                assert case_id in case_ids_val
+                val_dataset.add(case)
+
+        test_dataset = Dataset(f"{dataset_id}_test")
+        for case_id in case_ids_test:
+            case = Case(case_id)
+            record = Record("record_0")
+            if os.path.isdir(labels_dir_test):
+                assert os.path.isfile(os.path.join(labels_dir_test, f"{case_id}{file_extension}"))
+                record.add(NiftiFileModality("label", os.path.join(labels_dir_test, f"{case_id}{file_extension}")))
+
+            for i, channel_path in enumerate(sorted(glob.glob(os.path.join(images_dir_test, f"{case_id}_*.nii.gz")))):
+                record.add(NiftiFileModality("channel_{:04}".format(i), channel_path))
+            
+            case.add(record)
+            test_dataset.add(case)
+
+        return train_dataset, val_dataset, test_dataset
+
 
 class Case(SortedDict):
     """A dictionary subclass that stores medical imaging data for a single case.
@@ -531,7 +630,7 @@ class Modality(object):
         An abstract method to load the modality data.
     """
 
-    def __init__(self, modality_id, modality_dir=None):
+    def __init__(self, modality_id, modality_dir=None, shape=None, affine=None):
         """Constructor of the Modality class.
 
         Parameters
@@ -544,6 +643,8 @@ class Modality(object):
 
         self.modality_id = modality_id
         self.modality_dir = modality_dir
+        self.shape = Sample.ln_to_l5(shape)
+        self.affine = Sample.aff_to_144(affine)
 
     def load(self):
         """An abstract method to load the modality data.
@@ -597,9 +698,8 @@ class ArrayModality(Modality):
             The affine transformation matrix for the modality data. Default is None.
         """
 
-        super(ArrayModality, self).__init__(modality_id)
+        super(ArrayModality, self).__init__(modality_id, shape=array.shape, affine=affine)
         self.array = array
-        self.affine = affine
 
     def load(self):
         """Returns a Sample object with the modality data and affine matrix.
@@ -634,7 +734,7 @@ class NiftyModality(Modality):
         Returns the loaded modality as a Sample object, which is an ndarray subclass with an associated affine.
     """
 
-    def __init__(self, modality_id, nifty):
+    def __init__(self, modality_id, nifty, caching="unchanged"):
         """Initializes a new instance of the NiftyModality class.
 
         Parameters:
@@ -645,8 +745,9 @@ class NiftyModality(Modality):
             NIfTI image object containing the modality data.
         """
 
-        super(NiftyModality, self).__init__(modality_id)
+        super(NiftyModality, self).__init__(modality_id, shape=nifty.shape, affine=nifty.affine)
         self.nifty = nifty
+        self.caching = caching
 
     def load(self):
         """Returns the loaded modality as a Sample object, which is an ndarray subclass with an associated affine.
@@ -657,7 +758,7 @@ class NiftyModality(Modality):
             ndarray subclass with an associated affine representing the loaded modality data.
         """
 
-        return Sample(self.nifty.get_fdata(caching="unchanged"), self.nifty.affine)
+        return Sample(self.nifty.get_fdata(caching=self.caching), self.affine)
 
 
 NiftiModality = NiftyModality
@@ -683,7 +784,7 @@ class NiftyFileModality(Modality):
     The NIfTI file should be 3D or 4D. In the case of a 4D NIfTI file, the fourth dimension is interpreted as the feature dimension.
     """
 
-    def __init__(self, modality_id, file_path):
+    def __init__(self, modality_id, file_path, caching="unchanged"):
         """Initialize a new `NiftyFileModality` object.
 
         Parameters
@@ -699,8 +800,11 @@ class NiftyFileModality(Modality):
             The new `NiftyFileModality` object.
         """
 
-        super(NiftyFileModality, self).__init__(modality_id, os.path.dirname(file_path))
+        nifty = nib.load(file_path)
+        super(NiftyFileModality, self).__init__(modality_id, os.path.dirname(file_path), shape=nifty.shape, affine=nifty.affine)
         self.file_path = file_path
+        self.nifty = nifty
+        self.caching = caching
 
     def load(self):
         """Load the modality data from the NIfTI file.
@@ -711,8 +815,8 @@ class NiftyFileModality(Modality):
             A Sample object representing the modality data.
         """
 
-        nii = nib.load(self.file_path)
-        return Sample(nii.get_fdata(caching="unchanged"), nii.affine)
+        # return Sample(self.nifty.get_fdata(caching=self.caching), self.affine)
+        return Sample(np.asarray(self.nifty.dataobj), self.affine)
 
 
 NiftiFileModality = NiftyFileModality
@@ -748,7 +852,7 @@ class NiftyFileMultiModality(Modality):
         Mode for combining the volumes.
     """
 
-    def __init__(self, modality_id, file_paths, axis=-1, mode="stack"):
+    def __init__(self, modality_id, file_paths, axis=-1, mode="stack", caching="unchanged"):
         """Initialize an instance of the `NiftyFileMultiModality` class.
 
         Args:
@@ -760,10 +864,17 @@ class NiftyFileMultiModality(Modality):
                 in the files will be concatenated along the specified axis. Defaults to 'stack'.
         """
 
-        super(NiftyFileMultiModality, self).__init__(modality_id)
+        assert axis in [0, -1], "For now only first and last axes are supported!"
+        niftys = [nib.load(file_path) for file_path in file_paths]
+        assert all([np.allclose(niftys[0].affine, nii.affine) for nii in niftys[1:]]), "Not all affines are equal!"
+        shape = np.concatenate([nii.shape for nii in niftys], axis=axis) if mode == "concat" else np.stack([nii.shape for nii in niftys], axis=axis)
+        affine = [niftys[0].affine] if mode == "concat" else [niftys[0].affine] * len(niftys)
+        super(NiftyFileMultiModality, self).__init__(modality_id, shape=shape, affine=affine)
+        self.niftys = niftys
         self.file_paths = file_paths
         self.axis = axis
         self.mode = mode
+        self.caching = caching
 
     def load(self):
         """Load the NIfTI files and return them as a `Sample` object.
@@ -779,15 +890,13 @@ class NiftyFileMultiModality(Modality):
             If any of the NIfTI files cannot be loaded.
         """
 
-        niis = [nib.load(file_path) for file_path in self.file_paths]
-        # assert all([np.allclose(niis[0].affine, nii.affine) for nii in niis[1:]]), "Not all affines are equal!"
         if self.mode == "concat":
-            array = np.concatenate([nii.get_fdata(caching="unchanged") for nii in niis], axis=self.axis)
+            array = np.concatenate([nii.get_fdata(caching=self.caching) for nii in self.niftys], axis=self.axis)
 
         else:
-            array = np.stack([nii.get_fdata(caching="unchanged") for nii in niis], axis=self.axis)
+            array = np.stack([nii.get_fdata(caching=self.caching) for nii in self.niftys], axis=self.axis)
 
-        return Sample(array, niis[0].affine)
+        return Sample(array, self.affine)
 
 
 NiftiFileMultiModality = NiftyFileMultiModality
@@ -816,14 +925,19 @@ class NiftyMultiModality(Modality):
         If the mode parameter is not "stack" or "concat".
     """
 
-    def __init__(self, modality_id, niftys, axis=-1, mode="stack"):
+    def __init__(self, modality_id, niftys, axis=-1, mode="stack", caching="unchanged"):
         """Initializes a new instance of the NiftyMultiModality class.
         """
 
-        super(NiftyMultiModality, self).__init__(modality_id)
+        assert axis in [0, -1], "For now only first and last axes are supported!"
+        assert all([np.allclose(niftys[0].affine, nii.affine) for nii in niftys[1:]]), "Not all affines are equal!"
+        shape = np.concatenate([nii.shape for nii in niftys], axis=axis) if mode == "concat" else np.stack([nii.shape for nii in niftys], axis=axis)
+        affine = [niftys[0].affine] if mode == "concat" else [niftys[0].affine] * len(niftys)
+        super(NiftyMultiModality, self).__init__(modality_id, shape=shape, affine=affine)
         self.niftys = niftys
         self.axis = axis
         self.mode = mode
+        self.caching = caching
 
     def load(self):
         """Load the NIfTI images in this modality and return them as a Sample.
@@ -834,15 +948,13 @@ class NiftyMultiModality(Modality):
             The NIfTI images in this modality as a Sample.
         """
 
-        niis = self.niftys
-        # assert all([np.allclose(niis[0].affine, nii.affine) for nii in niis[1:]]), "Not all affines are equal!"
         if self.mode == "concat":
-            array = np.concatenate([nii.get_fdata(caching="unchanged") for nii in niis], axis=self.axis)
+            array = np.concatenate([nii.get_fdata(caching=self.caching) for nii in self.niftys], axis=self.axis)
 
         else:
-            array = np.stack([nii.get_fdata(caching="unchanged") for nii in niis], axis=self.axis)
+            array = np.stack([nii.get_fdata(caching=self.caching) for nii in self.niftys], axis=self.axis)
 
-        return Sample(array, niis[0].affine)
+        return Sample(array, self.affine)
 
 
 NiftiMultiModality = NiftyMultiModality
@@ -880,7 +992,7 @@ class ImageFileModality(Modality):
     PIL.Image.convert : Method used to convert the image to a numpy array.
     """
 
-    def __init__(self, modality_id, file_path, **kwargs):  # check documentation of Image.convert for **kwargs: e.g. mode, which can be "1" (binary), "RGB" (color), "L" grayscale
+    def __init__(self, modality_id, file_path, shape=None, affine=None, **kwargs):  # check documentation of Image.convert for **kwargs: e.g. mode, which can be "1" (binary), "RGB" (color), "L" grayscale
         """Initialize the ImageFileModality.
 
         Parameters
@@ -893,7 +1005,7 @@ class ImageFileModality(Modality):
             Additional keyword arguments passed to PIL's `Image.convert` function.
         """
 
-        super(ImageFileModality, self).__init__(modality_id, os.path.dirname(file_path))
+        super(ImageFileModality, self).__init__(modality_id, os.path.dirname(file_path), shape=shape, affine=affine)
         self.file_path = file_path
         self.kwargs = kwargs
 
