@@ -712,6 +712,22 @@ class SampleInput(_SampleInput):
         return _SampleInput(samples, output_shapes, **kwargs)()
 
 
+class Identity(Transformer):
+    def __init__(self, **kwargs):
+        super(Identity, self).__init__(**kwargs)
+
+    def _update_idx(self, idx):
+        for idx_, sample in enumerate(self.connections[idx][0]):
+            self.outputs[idx][idx_] = sample
+
+    def _calculate_output_shape_at_idx(self, idx):
+        assert len(self.connections[idx]) == 1, "This transformer accepts only a single connection at every idx."
+        return self.connections[idx][0].shapes
+
+    def _randomize(self):
+        pass
+
+
 class Buffer(Transformer):
     """Buffer class to concatenate incoming samples along the specified axis and buffer them until the buffer is full.
 
@@ -749,7 +765,7 @@ class Buffer(Transformer):
         assert "n" not in kwargs, "A Buffer does not accept n. It just buffers so it cannot create n samples from 1 input."
         super(Buffer, self).__init__(n=1, **kwargs)
         self.buffer_size = buffer_size
-        assert axis in [0, 4, -1]
+        # assert axis in [0, 4, -1]
         self.axis = axis if axis != -1 else 4
         self.drop_remainder = drop_remainder
         self.buffered_outputs = None
@@ -796,7 +812,7 @@ class Buffer(Transformer):
         StopIteration
             If `drop_remainder` is True and there are not enough samples to fill the buffer completely.
         """
-
+        initial_sample_ids = [transformer.sample_id for transformer in Connection.trace([connection for idx in self.active_indices for connection in self.connections[idx]], only_active=True)[0]]
         self.buffered_outputs = [[[sample for sample in self.connections[idx][0]]] if idx in self.active_indices else None for idx in range(len(self.outputs))]
         while self.buffer_size is None or len(self.buffered_outputs[0]) < self.buffer_size:
             try:
@@ -811,8 +827,9 @@ class Buffer(Transformer):
             raise StopIteration
 
         else:
-            for transformer in Connection.trace([connection for connections in self.connections for connection in connections], only_active=True)[0]:
-                transformer.sample_id = self.sample_id
+            for i, transformer in enumerate(Connection.trace([connection for idx in self.active_indices for connection in self.connections[idx]], only_active=True)[0]):
+                if initial_sample_ids[i] != transformer.sample_id:
+                    transformer.sample_id = self.sample_id
 
 
 class Group(Transformer):
@@ -981,12 +998,12 @@ class Mean(Transformer):
         """
 
         super(Mean, self).__init__(**kwargs)
-        assert axis in [4, -1]
+        # assert axis in [4, -1]
         self.axis = axis if axis != -1 else 4
 
     def _update_idx(self, idx):
         for idx_, sample in enumerate(self.connections[idx][0]):
-            self.outputs[idx][idx_] = sample.mean(axis=self.axis, keepdims=True)
+            self.outputs[idx][idx_] = sample.mean(axis=self.axis, keepdims=True) if self.axis != 0 else Sample(np.mean(sample, axis=0, keepdims=True), affine=sample.affine[0])
 
     def _calculate_output_shape_at_idx(self, idx):
         assert len(self.connections[idx]) == 1, "This transformer accepts only a single connection at every idx."
@@ -1265,7 +1282,7 @@ class NormalizeIndividual(Transformer):
 
 
 class NormalizeMask(Transformer):
-    def __init__(self, reference_connection, axis=(1, 2, 3), mean_shift=0, std_shift=0, mean_scale=1, std_scale=0, **kwargs):
+    def __init__(self, reference_connection, axis=(1, 2, 3), mean_shift=0, std_shift=0, mean_scale=0, std_scale=0, **kwargs):
         super(NormalizeMask, self).__init__(extra_connections=reference_connection, **kwargs)
         self.reference_connection = reference_connection
         self.axis = axis
@@ -1571,6 +1588,9 @@ class Subsample(Transformer):
 class Flip(Transformer):
     def __init__(self, flip_probabilities=(0.5, 0.5, 0.5), **kwargs):
         super(Flip, self).__init__(**kwargs)
+        if isinstance(flip_probabilities, str):
+            assert flip_probabilities == "all" and self.n == 8
+
         self.flip_probabilities = flip_probabilities
         self.flip_state = None
 
@@ -1586,7 +1606,36 @@ class Flip(Transformer):
         return self.connections[idx][0].shapes
 
     def _randomize(self):
-        self.flip_state = [-1 if random.random() < flip_probability else 1 for flip_probability in self.flip_probabilities]
+        if isinstance(self.flip_probabilities, str):
+            if self.n_ == 0:
+                self.flip_state = [1, 1, 1]
+            
+            elif self.n_ == 1:
+                self.flip_state = [-1, 1, 1]
+            
+            elif self.n_ == 2:
+                self.flip_state = [1, -1, 1]
+            
+            elif self.n_ == 3:
+                self.flip_state = [-1, -1, 1]
+            
+            elif self.n_ == 4:
+                self.flip_state = [1, 1, -1]
+            
+            elif self.n_ == 5:
+                self.flip_state = [-1, 1, -1]
+            
+            elif self.n_ == 6:
+                self.flip_state = [1, -1, -1]
+            
+            elif self.n_ == 7:
+                self.flip_state = [-1, -1, -1]
+
+            else:
+                raise ValueError
+                        
+        else:
+            self.flip_state = [-1 if random.random() < flip_probability else 1 for flip_probability in self.flip_probabilities]
 
 
 class GaussianNoise(Transformer):
@@ -1722,10 +1771,10 @@ class AffineDeformation(Transformer):
             self.backward_affine = transformations.get_affine_matrix(
                 I_shape=self.reference_connection[0].shape[1:4],
                 voxel_size=self.voxel_size,
-                shear=[np.random.normal(0, w) if self.width_as_std[0] else random.uniform(-w, w) for w in self.shear_window_width],
-                rotation=[np.random.normal(0, w) if self.width_as_std[1] else random.uniform(-w, w) for w in self.rotation_window_width],
-                translation=[np.random.normal(0, w) if self.width_as_std[2] else random.uniform(-w, w) for w in self.translation_window_width],
-                scaling=[1 + (np.random.normal(0, w) if self.width_as_std[3] else random.uniform(-w, w)) for w in self.scaling_window_width],
+                shear=[np.random.normal(0, w) if self.width_as_std[0] else random.uniform(-w, w) for w in self.shear_window_width] * (3 if len(self.shear_window_width) == 1 else 1),
+                rotation=[np.random.normal(0, w) if self.width_as_std[1] else random.uniform(-w, w) for w in self.rotation_window_width] * (3 if len(self.rotation_window_width) == 1 else 1),
+                translation=[np.random.normal(0, w) if self.width_as_std[2] else random.uniform(-w, w) for w in self.translation_window_width] * (3 if len(self.translation_window_width) == 1 else 1),
+                scaling=[1 + (np.random.normal(0, w) if self.width_as_std[3] else random.uniform(-w, w)) for w in self.scaling_window_width] * (3 if len(self.scaling_window_width) == 1 else 1),
             )
 
         else:
@@ -1818,7 +1867,18 @@ class RandomCrop(Crop):
     def _randomize(self):
         if self.n_ == 0:
             assert all([np.array_equal(self.reference_connection[0].shape[1:4], sample.shape[1:4]) for connection in self.connections for sample in connection[0] if sample is not None])
-            if random.random() < self.nonzero:
+            if self.nonzero == "p":
+                values, counts = np.unique(self.reference_connection[0], return_counts=True)
+                p = 1 / counts / np.sum(1 / counts)
+                self.coordinates, coordinates_dict = [], {}
+                for v in np.random.choice(values, size=self.ncrops, p=p):
+                    if v not in coordinates_dict:
+                        coordinates_dict[v] = np.where(self.reference_connection[0] == v)
+
+                    c = random.randrange(len(coordinates_dict[v][0]))
+                    self.coordinates.append((coordinates_dict[v][1][c], coordinates_dict[v][2][c], coordinates_dict[v][3][c]))
+                    
+            elif random.random() < self.nonzero:
                 self.coordinates = list(zip(*np.nonzero(np.any(self.reference_connection[0] != 0, axis=(0, -1)))))
                 if len(self.coordinates) > 0 and self.ncrops is not None:
                     self.coordinates = [random.choice(self.coordinates) for _ in range(self.ncrops)]
