@@ -1,3 +1,8 @@
+"""The transformations module provides functions for various data transformations and augmentations, such as affine transformations, elastic deformations, orthogonalization of image directions, and registration.
+
+These functions are implemented using the SimpleITK and NumPy libraries, and can be used to preprocess or augment medical image data for various applications, such as computer-aided diagnosis, image segmentation, or deep learning.
+"""
+
 import os
 import numpy as np
 import nibabel as nib
@@ -16,21 +21,32 @@ from .conversions import nii_to_sitk, sitk_to_nii
 
 
 def downsample_array(input_array, window_sizes):
-    """
-    Performs a moving average filter across an array with stride equal to window size.
+    """Downsamples an input array using a sliding window and averaging over the window.
+
+    This function performs a sliding window with a stride equal to the window size over each dimension of the input array with the specified window size, and
+    then averages over the window to produce a single output value. If the size of the input array is not divisible by the
+    window size, the output array will be truncated to the largest integer size that is divisible by the window size.
 
     Parameters
     ----------
-    input_array: np.ndarray
-        The array to be downsampled.
-    window_sizes: int, tuple, list
-        The window size to be used for the moving average filter. Must be specified as integers > 0 or a iterable as such.
+    input_array : numpy.ndarray
+        Input array to be downsampled.
+    window_sizes : tuple or list of int
+        Window size for each dimension of the input array. If only one value is specified, the same window size is used
+        for all dimensions.
 
     Returns
     -------
-    output_array: np.ndarray
-        Downsampled version of the input array.
+    output_array : numpy.ndarray
+        Downsampled array.
+
+    Raises
+    ------
+    AssertionError
+        If `window_sizes` is not a tuple or list, or if not all dimensions in the input array have a corresponding window
+        size being specified, or if any of the window sizes are not >= 1 integers.
     """
+
     if not isinstance(window_sizes, (tuple, list)):
         window_sizes = [window_sizes] * len(input_array.shape)
 
@@ -49,31 +65,28 @@ def downsample_array(input_array, window_sizes):
 
 
 def resample(input_nii, output_zooms, order=3, prefilter=True, reference_nii=None):
-    """
-    This function resamples a Nifti image to the desired zooms.
+    """Resamples a Nifti image to have new voxel sizes defined by `output_zooms`.
 
     Parameters
     ----------
-    input_nii: nib.Nifti1Image
-        The input Nifti image to be resampled.
-    output_zooms: tuple, list
-        The desired output zooms. It must be a tuple or list of same length as the number of dimensions in the original Nifti image. None can be used to specify that a certain dimension does not need to be resampled.
-    order: int, str
-        Order of the interpolation that will be used. See supported orders in NumPy's zoom function (order=0, 1, 2, 3). If you choose order='mean' instead, than it will be downsampling with a moving average window that gets the effective output zooms as close as possible to the requested output zooms.
-    prefilter: bool
-        Use gaussian prefilter when downsampling (https://stackoverflow.com/questions/35340197/box-filter-size-in-relation-to-gaussian-filter-sigma).
-    reference_nii: nib.Nifti1Image
-        E.g. when doing upsampling back to an original image space we want to make sure the image sizes are consistent. By giving a reference Nifti image, we crop or pad with zeros where necessary.
+    input_nii : nibabel.nifti1.Nifti1Image
+        The input Nifti image to resample.
+    output_zooms : tuple
+        The desired voxel size for the resampled Nifti image. It must be a tuple or list of same length as the number of dimensions in the original Nifti image. None can be used to specify that a certain dimension does not need to be resampled.
+    order : int or str, optional
+        The order of interpolation (0 to 5) or the string 'mean' to perform a downsampling based on mean (with a moving average window that gets the effective output zooms as close as possible to the requested output zooms). Default is 3.
+    prefilter : bool, optional
+        Whether to apply a Gaussian filter to the input data before resampling (https://stackoverflow.com/questions/35340197/box-filter-size-in-relation-to-gaussian-filter-sigma). Default is True.
+    reference_nii : nibabel.nifti1.Nifti1Image or None, optional
+        If not None, the resampled Nifti image will have the same dimensions as `reference_nii`. Default is None.
+        E.g., when doing upsampling back to an original image space we want to make sure the image sizes are consistent. By giving a reference Nifti image, we crop or pad with zeros where necessary.
 
     Returns
     -------
-    output_nii: nib.Nifti1Image
-        Resampled version of the original Nifti image.
-
-    See Also
-    --------
-    scipy.ndimage.zoom: The core function that is used under the hood.
+    output_nii : nibabel.nifti1.Nifti1Image
+        The resampled Nifti image.
     """
+
     input_zooms = input_nii.header.get_zooms()
     assert len(input_zooms) == len(output_zooms), "Number of dimensions mismatch."
     assert np.allclose(input_zooms[:3], np.linalg.norm(input_nii.affine[:3, :3], 2, axis=0)), "Inconsistency (we only support voxel size = voxel distance) in affine and zooms (spatial) of input Nifti image."
@@ -110,29 +123,29 @@ def resample(input_nii, output_zooms, order=3, prefilter=True, reference_nii=Non
 
 
 def crop(I, S_size, coordinates=None, subsample_factors=None, default_value=0, prefilter=None):
-    """
-    This function can be used to extract an array of a specific size centered on a specific coordinate from the input array.
+    """Crop an input array I to size S_size centered on coordinates in I with optional subsampling.
 
     Parameters
     ----------
-    I: np.ndarray
-        The input array. Can be of any number of dimensions, but the number of axes specified in the other arguments must (i) all be the same and (ii) less than or equal to the number of image dimensions. Remaining trailing dimensions will not be cropped.
-    S_size: tuple, list
-        The requested output array size of the cropped volume.
-    coordinates: None, tuple, list
-        Around what coordinate in the input array the array will be cropped. When None, the center coordinate will be used. None can also be used for a specific axis to specify that the center coordinate along that axis must be used.
-    subsample_factors: None, tuple, list
-        If the cropped array must be subsampled, one can specify an integer subsample factor for each axis. None is used to denote a subsample factor of 1.
-    default_value: int
-        What value to pad with outside the input array, if np.nan a nearest interpolation is done at the borders.
-    prefilter: None, str
-        One of "gaussian" and "uniform", the interpolation used in case prefilter is not None (the "uniform" prefilter is implemented with downsample_array function).
+    I : np.ndarray
+        Input array. Must be at least as large as S_size in each dimension.
+    S_size : tuple, list
+        Output array size of the cropped volume.
+    coordinates : tuple, list or None, optional
+        Indices in I to center the crop. If None, the center of I is used.
+    subsample_factors : tuple, list of int or None, optional
+        Subsampling factors for each requested segment dimension.
+    default_value : int or np.nan, optional
+        Value to fill outside of I. If np.nan, a nearest-neighbor interpolation is used to fill the edges.
+    prefilter : {'gaussian', 'uniform'} or None, optional
+        Filter to use during interpolation. Uniform filter uses the `downsample_array` function.
 
     Returns
     -------
-    S: np.ndarray
-        The output array crop.
+    np.ndarray
+        Cropped array.
     """
+
     assert I.ndim >= len(S_size), "Number of image dimensions must be equal to or larger than the number of requested segment sizes. Extra (trailing) dimensions will not be cropped."
     if coordinates is None:
         coordinates = [s // 2 for s in I.shape]  # cropping happens around the center
@@ -198,25 +211,27 @@ def crop(I, S_size, coordinates=None, subsample_factors=None, default_value=0, p
 
 
 def put(I, S, coordinates=None, subsample_factors=None):
-    """
-    This function can be used to put back a segment centered on a specific coordinate in the input array (reverse of crop function).
+    """This function can be used to put back a segment centered on a specific coordinate in the input array (reverse of crop function).
 
     Parameters
     ----------
-    I: np.ndarray
+    I : np.ndarray
         The input array in which to put back the segment S. Can be of any number of dimensions.
-    S: np.ndarray
+    S : np.ndarray
         The segment array to put back in I. Same number of dimensions as I.
-    coordinates: None, tuple, list
-        Around what coordinate in the input array the segment array will be put back. When None, the center coordinate in I will be used. None can also be used for a specific axis to specify that the center coordinate along that axis must be used.
-    subsample_factors: None, tuple, list
-        If the segment array was subsampled, one can specify an integer subsample factor for each axis. None is used to denote a subsample factor of 1.
+    coordinates : None or tuple or list
+        Around what coordinate in the input array the segment array will be put back. When None, the center coordinate in I will be used.
+        None can also be used for a specific axis to specify that the center coordinate along that axis must be used.
+    subsample_factors : None or tuple or list
+        If the segment array was subsampled, one can specify an integer subsample factor for each axis.
+        None is used to denote a subsample factor of 1.
 
     Returns
     -------
-    I: np.ndarray
+    I : np.ndarray
         The output array I in which the segment S is now put.
     """
+
     assert I.ndim == S.ndim, "Number of image dimensions must be equal to number of segment dimensions."
     if coordinates is None:
         coordinates = [s // 2 for s in I.shape]  # putting happens around the center
@@ -270,30 +285,32 @@ def put(I, S, coordinates=None, subsample_factors=None):
 def get_affine_matrix(I_shape, voxel_size=(1, 1, 1), reflection=(1, 1, 1), shear=(0, 0, 0), rotation=(0, 0, 0), translation=(0, 0, 0), scaling=(1, 1, 1), return_forward_affine=False):
     """
     Apply a certain deformation to an image, using the center of the image as origin for the scaling and rotation, and afterwards an optional translation as well.
-    ---->   Therefor voxel space has its origin at the corner of the image (x = y = z = 0) and the units of the axes are the pixels/voxels.
-            In world space the origin is at the center of the image and the axes are scaled according to the voxel size. It is there that the transformation is defined.
 
-          --------- p_v voxel space p_v' <--------
-         |                                        |
-         | W                                      | W-1
-         v                                        |
-    world space p_w       == T_w ==>         world space' p_w'
+    Parameters
+    ----------
+    I_shape : tuple
+        The shape of the image in the form of (x, y, z). The image is centered using this shape.
+    voxel_size : tuple, optional
+        The voxel size in the form of (x, y, z). Default is (1, 1, 1).
+    reflection : tuple, optional
+        The reflection along each axis in the form of (x, y, z). Default is (1, 1, 1).
+    shear : tuple, optional
+        The shear angle in radians for each axis in the form of (x, y, z). Default is (0, 0, 0).
+    rotation : tuple, optional
+        The rotation angles in radians for each axis in the form of (x, y, z). Default is (0, 0, 0).
+    translation : tuple, optional
+        The translation along each axis in the form of (x, y, z). Default is (0, 0, 0).
+    scaling : tuple, optional
+        The scaling along each axis in the form of (x, y, z). Default is (1, 1, 1).
+    return_forward_affine : bool, optional
+        Whether to return the forward or inverse transformation matrix. Default is False (i.e., returns the inverse).
 
-    Step 1:
-        The image is centered (T2c = Translation to center) and then the axes are scaled according to the voxel size (S2w = Scaled to world (real size)) => W = S2w T2c
-    Step 2:
-        The image as such is "augmented": rotated (Rw = rotated in world), scaled (Sw = scaled in world) and translated (Tw = translated in ) in that order => T_w = Tw Sw Rw
-    Step 3:
-        The image is put back on its original place (T2c-1) with the original voxel size (S2w-1) => W-1 = T2c-1 S2w-1
-    This means the transformation matrix = matrix-1 for the augmentation is:
-        matrix-1 = W-1 T_w W = T2c-1 . S2w-1 . Tw . Sw . Rw . S2w . T2c
-    Because the function takes the transformation matrix from deformed space to original space, we take the inverse tranformation matrix = matrix as input to the function.
-
-    p_w         = W p_v
-    p_w'        = T_w W p_v
-    p_v'        = W-1 T_w W p_v
-                = T2c-1 S2w-1 Tw Sw Rw S2w T2c p_v
+    Returns
+    -------
+    affine : np.ndarray
+        The affine transformation matrix that describes the specified transformation.
     """
+
     Tw, Sw, Rw, S2w, T2c, SHw, RFw = [np.eye(4) for _ in range(7)]
     T2c[:-1, -1] = - np.array(I_shape) / 2
     S2w[:-1, :-1] = np.diag(voxel_size)
@@ -310,35 +327,104 @@ def get_affine_matrix(I_shape, voxel_size=(1, 1, 1), reflection=(1, 1, 1), shear
 
 
 def affine_deformation(I, affine, output_shape=None, order=1, cval=0, oversampling_factors=(1, 1, 1)):
+    """Apply an affine deformation to the input array I.
+
+    Parameters
+    ----------
+    I: np.ndarray
+        The input array to be deformed.
+    affine: np.ndarray
+        The 4x4 affine transformation matrix specifying the transformation to apply.
+    output_shape: tuple, None
+        The shape of the output array. If None, the output shape will be the same as the input array I.
+    order: int
+        The order of the interpolation used in the transformation. Can be 0, 1, 2, 3, 4, or 5.
+    cval: int
+        The value to fill in for points outside the boundaries of the input array. Default is 0.
+    oversampling_factors: tuple
+        The oversampling factors in each dimension. Default is (1, 1, 1).
+
+    Returns
+    -------
+    deformed_array: np.ndarray
+        The deformed array I after applying the specified affine transformation.
+    """
+
     return aff_transform(I, np.round(affine, 5), output_shape=output_shape or I.shape, trilin=order, cval=cval, os0=oversampling_factors[0], os1=oversampling_factors[1], os2=oversampling_factors[2])
 
 
 def get_deformation_field(I_shape, shift=(2, 2, 2), nsize=(30, 30, 30), npad=(5, 5, 5), std=(6, 6, 6)):
+    """Generate an elastic deformation field.
+
+    Parameters
+    ----------
+    I_shape: tuple
+        The shape of the image for which to generate the deformation field.
+    shift: tuple
+        The maximum shift in each dimension.
+    nsize: tuple
+        The number of points in each dimension for which to calculate the deformation field.
+    npad: tuple
+        The padding in each dimension for which to calculate the deformation field.
+    std: tuple
+        The standard deviation of the Gaussian smoothing kernel in each dimension.
+
+    Returns
+    -------
+    deformation_field: np.ndarray
+        The elastic deformation field as a 4D numpy array.
+    """
+
     deformation_field = np.zeros(tuple(I_shape) + (3,))
     deformation_field[..., 0], deformation_field[..., 1], deformation_field[..., 2] = random_deformation_field(I_shape, shift=shift, n=nsize, npad=npad, gaussian_std=std)
     return deformation_field
 
 
 def elastic_deformation(I, deformation_field, cval=0, order=1):
+    """Apply an elastic deformation to the input array I.
+
+    Parameters
+    ----------
+    I: np.ndarray
+        The input array to be deformed.
+    deformation_field: np.ndarray
+        The 4D elastic deformation field specifying the deformation to apply.
+    cval: int
+        The value to fill in for points outside the boundaries of the input array. Default is 0.
+    order: int
+        The order of the interpolation used in the transformation. Can be 0, 1, 2, 3, 4, or 5.
+
+    Returns
+    -------
+    deformed_array: np.ndarray
+        The deformed array I after applying the specified elastic deformation.
+    """
+
     return backward_3d_warp(I, deformation_field[..., 0], deformation_field[..., 1], deformation_field[..., 2], trilin=order, cval=cval)
 
 
 def orthogonalize(input_nii, invalid_value):
-    """
-    Orthogonalize by having a new z-axis that is orthogonal to the x and y axis. Keeps the dimensions of the image.
+    """Orthogonalize by having a new z-axis that is orthogonal to the x and y axis.
+
+    This function takes a nibabel.Nifti1Image as input, which is a 3D image with x, y, and z axes.
+    The function then finds a new z-axis that is orthogonal to the x and y axes of the input image, keeping the dimensions of the image the same.
+    It returns the orthogonalized image as another nibabel.Nifti1Image.
+    The invalid_value parameter is used to specify the value to fill in outside of the image.
+    Keeps the dimensions of the image.
 
     Parameters
     ----------
-    input_nii: nib.Nifti1Image
+    input_nii : nib.Nifti1Image
         Input image.
-    invalid_value: float
+    invalid_value : float
         Outside image values.
 
     Returns
     -------
-    output_nii: nib.Nifti1Image
+    output_nii : nib.Nifti1Image
         Orthogonalized input image.
     """
+
     image = nii_to_sitk(input_nii)
     direction_matrix = np.array(image.GetDirection()).reshape((3, 3))
     z_direction = direction_matrix[:, 2]
@@ -358,27 +444,23 @@ def orthogonalize(input_nii, invalid_value):
 
 
 def registration_quality(fixed_array, moved_array, mask_array=None):
-    """
-    Checking registration quality of two arrays, optionally using a mask.
+    """Computes the quality of registration between two arrays using the correlation coefficient.
 
     Parameters
     ----------
     fixed_array: np.ndarray
-        First array.
+        The reference array to which the `moved_array` is registered.
     moved_array: np.ndarray
-        Second array.
-    mask_array: np.ndarray
-        Mask array (optional).
+        The array that has been registered to the `fixed_array`.
+    mask_array: None or np.ndarray, optional
+        A mask array with 0 values where the corresponding voxels should be ignored in the computation.
 
     Returns
     -------
     correlation_coefficient: float
-        Correlation coefficient of the two, optionally masked, arrays.
-
-    See Also
-    --------
-    np.corrcoef: The NumPy function that is used under the hood.
+        The correlation coefficient between the flattened `fixed_array` and `moved_array`.
     """
+
     if mask_array is not None:
         fixed_array = fixed_array[mask_array]
         moved_array = moved_array[mask_array]
@@ -388,50 +470,38 @@ def registration_quality(fixed_array, moved_array, mask_array=None):
 
 
 def move(moving_nii, fixed_nii=None, transform_parameter_map=None, fixed_nii_mask=None, moving_nii_mask=None, parameter_map=None, initial_transform_parameter_map=None):
-    """
-    Function to move a 'moving image'. When a 'fixed image' is specified, this function is an actual registration. Otherwise, it will be simply applying a transformation to the image.
-    To install Simple ITK with Simple Elastix there are two options:
-    1) Following the installation instructions from https://simpleelastix.readthedocs.io/index.html with some small tweaks:
-        $ git clone https://github.com/SuperElastix/SimpleElastix
-        $ mkdir build
-        $ cd build
-        $ cmake -DCMAKE_CXX_COMPILER:STRING=/usr/bin/clang++ -DCMAKE_C_COMPILER:STRING=/usr/bin/clang -DWRAP_JAVA:BOOL=OFF -DWRAP_LUA:BOOL=OFF -DWRAP_R:BOOL=OFF -DWRAP_RUBY:BOOL=OFF -DWRAP_TCL:BOOL=OFF -DSimpleITK_PYTHON_USE_VIRTUALENV:BOOL=OFF ../SimpleElastix/SuperBuild
-        $ make -j4
-        Then activate your virtual environment and do the following (make sure there is not yet any Simple ITK version installed in this environment!)
-        $ {BUILD_DIRECTORY}/SimpleITK-build/Wrapping/Python
-        $ python Packaging/setup.py install
-    2) Simply installing the simpleitk-elastix package via PIP in your virtual environment:
-        $ pip install simpleitk-elastix
+    """Apply a registration or a transform to a moving image using SimpleElastix.
 
     Parameters
     ----------
     moving_nii: nib.Nifti1Image
-        'moving image' Nifti.
-    fixed_nii: None, nib.Nifti1Image
-       'fixed image' Nifti. When specified, a registration is done.
-    transform_parameter_map: None, dict, list of dicts
-        A dictionary, or list of dictionaries, with the transformation parameter map(s) to be used to move the 'moving image'. When specified, a simple move of the 'moving image' will be done.
-    fixed_nii_mask: None, nib.Nifti1Image
-        'fixed image mask' Nifti. Only useful when doing a registration.
-    moving_nii_mask: None, nib.Nifti1Image
-        'moving image mask' Nifti. Only useful when doing a registration.
-    parameter_map: None, str, dict, sitk.ParameterMap
-        The type of registration that needs to be performed.
-    initial_transform_parameter_map: None, sitk.ParameterMap
-        Optional initial parameter map to be used.
+        The moving image to apply the transformation to.
+    fixed_nii: None or nib.Nifti1Image
+        If specified, the function will do registration of the moving image with the fixed image using SimpleElastix.
+    transform_parameter_map: None or sitk.Transform, dict, or sitk.ParameterMap
+        If no fixed image is specified, the parameter map (file, dict or SimpleITK object) specifying the transformation is applied to the moving image.
+    fixed_nii_mask: None or nib.Nifti1Image
+        If fixed image is specified, use this image as a mask for the fixed image.
+    moving_nii_mask: None or nib.Nifti1Image
+        If fixed image is specified, use this image as a mask for the moving image.
+    parameter_map: None, "default", "elastic", "translation", "rigid", "affine", sitk.ParameterMap or dict
+        If fixed image is specified, use this SimpleElastix parameter map for registration. Default value is "default".
+    initial_transform_parameter_map: None or sitk.ParameterMap
+        If specified, this initial transformation parameter map is used for the registration.
 
     Returns
     -------
-    moved_nii: nib.Nifti1Image
-        'moved image' Nifti.
-    transform_parameter_map: dict
-        The transformation parameter map that was used. In case of a requested registration this is the resulting transformation for the registration. Otherwise, this is simply the inputted transformation parameter map.
+    moved_nii: None or nib.Nifti1Image
+        The output Nifti image with the transformation applied. Is None if the transformation is unsuccessful.
+    transform_parameter_map: None or sitk.ParameterMap
+        The transform parameter map. Is None if the transformation is unsuccessful.
 
     See Also
     --------
     sitk.ElastixImageFilter: Class that is used under the hood to perform a registration.
     sitk.TransformixImageFilter: Class that is used under the hood to perform a move.
     """
+
     assert hasattr(sitk, "ElastixImageFilter"), "Please also install Simple Elastix (see documentation of this move function for installation instructions)."
     if fixed_nii is not None:
         assert transform_parameter_map is None, "When a fixed image is specified a registration is done and no transformation map will be used."
